# -*- coding: utf-8 -*-
# ~ Jesse K. Rubin ~ Pretty Useful Python
from __future__ import division
from __future__ import print_function
from __future__ import with_statement

from pupy.savings_n_loads import ljson
from pupy.savings_n_loads import load_jasm
from pupy.savings_n_loads import loads
from pupy.savings_n_loads import lpak
from pupy.savings_n_loads import save_jasm
from pupy.savings_n_loads import savings
from pupy.savings_n_loads import sjson
from pupy.savings_n_loads import spak

try:
    range
except NameError:
    range = xrange
